import logging
from _typeshed import Incomplete
from collections.abc import Callable

from authlib.oauth1 import AuthorizationServer as _AuthorizationServer, OAuth1Request

log: logging.Logger

class AuthorizationServer(_AuthorizationServer):
    app: Incomplete
    query_client: Incomplete
    token_generator: Incomplete
    def __init__(self, app=None, query_client=None, token_generator=None): ...
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def init_app(self, app, query_client=None, token_generator=None): ...
    def register_hook(self, name, func) -> None: ...
    def create_token_generator(self, app) -> Callable[[], dict[str, str]]: ...
    def get_client_by_id(self, client_id): ...
    def exists_nonce(self, nonce, request): ...
    def create_temporary_credential(self, request): ...
    def get_temporary_credential(self, request): ...
    def delete_temporary_credential(self, request): ...
    def create_authorization_verifier(self, request): ...
    def create_token_credential(self, request): ...
    def check_authorization_request(self) -> OAuth1Request: ...
    def create_authorization_response(self, request=None, grant_user=None): ...
    def create_token_response(self, request=None): ...
    def create_oauth1_request(self, request) -> OAuth1Request: ...
    def handle_response(self, status_code, payload, headers): ...
