from _typeshed import Incomplete

from authlib.oauth2 import AuthorizationServer as _AuthorizationServer
from authlib.oauth2.rfc6750 import BearerTokenGenerator

from .requests import FlaskJsonRequest, FlaskOAuth2Request

class AuthorizationServer(_AuthorizationServer):
    def __init__(self, app=None, query_client=None, save_token=None) -> None: ...
    def init_app(self, app, query_client=None, save_token=None) -> None: ...
    scopes_supported: Incomplete
    def load_config(self, config) -> None: ...
    def query_client(self, client_id): ...
    def save_token(self, token, request): ...
    def get_error_uri(self, request, error): ...
    def create_oauth2_request(self, request) -> FlaskOAuth2Request: ...
    def create_json_request(self, request) -> FlaskJsonRequest: ...
    def handle_response(self, status_code, payload, headers): ...
    def send_signal(self, name, *args, **kwargs) -> None: ...
    def create_bearer_token_generator(self, config) -> BearerTokenGenerator: ...

def create_token_expires_in_generator(expires_in_conf=None): ...
def create_token_generator(token_generator_conf, length: int = 42): ...
