from _typeshed import Incomplete

from authlib.oauth1 import ResourceProtector as _ResourceProtector

class ResourceProtector(_ResourceProtector):
    client_model: Incomplete
    token_model: Incomplete
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def __init__(self, client_model, token_model) -> None: ...
    def get_client_by_id(self, client_id): ...
    def get_token_credential(self, request): ...
    def exists_nonce(self, nonce, request) -> bool: ...
    def acquire_credential(self, request): ...
    def __call__(self, realm=None): ...
