from typing import Any

class GeneratedProtocolMessageType(type):
    def __new__(cls, name: str, bases: tuple[type, ...], dictionary: dict[str, Any]) -> GeneratedProtocolMessageType: ...
    def __init__(cls, name: str, bases: tuple[type, ...], dictionary: dict[str, Any]) -> None: ...
