from _typeshed import Incomplete
from typing import Final

__version__: Final[str]

def bezierArc(
    x1, y1, x2, y2, startAng: int = 0, extent: int = 90
) -> list[tuple[Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete, Incomplete]]: ...
