from win32.lib.pywintypes import error as error

def GetHandle() -> int: ...
def GetTracer(): ...
def InitRead() -> None: ...
def InitWrite() -> None: ...
def TermRead() -> None: ...
def TermWrite() -> None: ...
def blockingread(milliSeconds: int = ..., /) -> str: ...
def flush() -> None: ...
def read() -> str: ...
def setprint() -> None: ...
def write(data: str, /) -> None: ...
