import abc

class AbstractMetricsReporter(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def init(self, metrics): ...
    @abc.abstractmethod
    def metric_change(self, metric): ...
    @abc.abstractmethod
    def metric_removal(self, metric): ...
    @abc.abstractmethod
    def configure(self, configs): ...
    @abc.abstractmethod
    def close(self): ...
