from types import TracebackType

import ibm_db

class Db2connect:
    def __init__(self, dsn: str, username: str, password: str) -> None: ...
    def __enter__(self) -> ibm_db.IBM_DBConnection: ...
    def __exit__(self, t: type[BaseException] | None, v: BaseException | None, tb: TracebackType | None) -> None: ...
