from _typeshed import Incomplete

log: Incomplete

class Heartbeat:
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    last_send: Incomplete
    last_receive: Incomplete
    last_poll: Incomplete
    last_reset: Incomplete
    heartbeat_failed: Incomplete
    def __init__(self, **configs) -> None: ...
    def poll(self) -> None: ...
    def sent_heartbeat(self) -> None: ...
    def fail_heartbeat(self) -> None: ...
    def received_heartbeat(self) -> None: ...
    def time_to_next_heartbeat(self): ...
    def should_heartbeat(self): ...
    def session_timeout_expired(self): ...
    def reset_timeouts(self) -> None: ...
    def poll_timeout_expired(self): ...
