from _typeshed import Incomplete

log: Incomplete

class Future:
    error_on_callbacks: bool
    is_done: bool
    value: Incomplete
    exception: Incomplete
    def __init__(self) -> None: ...
    def succeeded(self): ...
    def failed(self): ...
    def retriable(self): ...
    def success(self, value): ...
    def failure(self, e): ...
    def add_callback(self, f, *args, **kwargs): ...
    def add_errback(self, f, *args, **kwargs): ...
    def add_both(self, f, *args, **kwargs): ...
    def chain(self, future): ...
