from _typeshed import Incomplete

from kafka.protocol.abstract import AbstractType

class Struct(AbstractType):
    SCHEMA: Incomplete
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def encode(cls, item): ...
    @classmethod
    def decode(cls, data): ...
    def get_item(self, name): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
