class Subscription:
    def __init__(self, metadata, group_instance_id) -> None: ...
    @property
    def version(self): ...
    @property
    def user_data(self): ...
    @property
    def topics(self): ...
    subscription = topics
    @property
    def group_instance_id(self): ...
    def encode(self): ...
    def __eq__(self, other): ...
