from collections.abc import Iterable

from .ast import Node

def serialize(nodes: Iterable[Node]) -> str: ...
def serialize_identifier(value: str) -> str: ...
def serialize_name(value: str) -> str: ...
def serialize_string_value(value: str) -> str: ...
def serialize_url(value: str) -> str: ...

BAD_PAIRS: set[tuple[str, str]]
