import abc

from kafka.metrics.stat import AbstractStat

class AbstractCompoundStat(AbstractStat, metaclass=abc.ABCMeta):
    def stats(self) -> None: ...

class NamedMeasurable:
    def __init__(self, metric_name, measurable_stat) -> None: ...
    @property
    def name(self): ...
    @property
    def stat(self): ...
