from kafka.metrics.stats.sampled_stat import AbstractSampledStat

class Count(AbstractSampledStat):
    def __init__(self) -> None: ...
    def update(self, sample, config, value, now) -> None: ...
    def combine(self, samples, config, now): ...
