class KafkaBytes(bytearray):
    def __init__(self, size) -> None: ...
    def read(self, nbytes=None): ...
    def write(self, data) -> None: ...
    def seek(self, idx) -> None: ...
    def tell(self): ...
