from _typeshed import Incomplete

from kafka.sasl.abc import SaslMechanism

log: Incomplete

class SaslMechanismSSPI(SaslMechanism):
    SASL_QOP_AUTH: int
    SASL_QOP_AUTH_INT: int
    SASL_QOP_AUTH_CONF: int
    auth_id: Incomplete
    def __init__(self, **config) -> None: ...
    def auth_bytes(self): ...
    def receive(self, auth_bytes) -> None: ...
    def is_done(self): ...
    def is_authenticated(self): ...
    def auth_details(self): ...
