from _typeshed import Incomplete
from datetime import tzinfo

class OffsetTzInfo(tzinfo):
    offset: Incomplete
    name: Incomplete
    def __init__(self, offset, name) -> None: ...
    def utcoffset(self, dt): ...
    def tzname(self, dt): ...
    def dst(self, dt): ...
    def __getinitargs__(self): ...
