from datetime import timedelta

from google.protobuf.duration_pb2 import Duration

def from_json_string(value: str) -> Duration: ...
def from_microseconds(micros: float) -> Duration: ...
def from_milliseconds(millis: float) -> Duration: ...
def from_nanoseconds(nanos: float) -> Duration: ...
def from_seconds(seconds: float) -> Duration: ...
def from_timedelta(td: timedelta) -> Duration: ...
def to_json_string(duration: Duration) -> str: ...
def to_microseconds(duration: Duration) -> int: ...
def to_milliseconds(duration: Duration) -> int: ...
def to_nanoseconds(duration: Duration) -> int: ...
def to_seconds(duration: Duration) -> int: ...
def to_timedelta(duration: Duration) -> timedelta: ...
