from _typeshed import Incomplete

from kafka.metrics.metrics_reporter import AbstractMetricsReporter

logger: Incomplete

class DictReporter(AbstractMetricsReporter):
    def __init__(self, prefix: str = "") -> None: ...
    def snapshot(self): ...
    def init(self, metrics) -> None: ...
    def metric_change(self, metric) -> None: ...
    def metric_removal(self, metric): ...
    def get_category(self, metric): ...
    def configure(self, configs) -> None: ...
    def close(self) -> None: ...
