class KafkaMetric:
    def __init__(self, metric_name, measurable, config) -> None: ...
    @property
    def metric_name(self): ...
    @property
    def measurable(self): ...

    @property
    def config(self): ...
    @config.setter
    def config(self, config) -> None: ...

    def value(self, time_ms=None): ...
