from typing import Final

FACILITY_CONTROL: Final = 0xA

def MAKE_SCODE(sev: int, fac: int, code: int) -> int: ...
def STD_CTL_SCODE(n: int) -> int: ...

CTL_E_ILLEGALFUNCTIONCALL: Final = -2146828283
CTL_E_OVERFLOW: Final = -2146828282
CTL_E_OUTOFMEMORY: Final = -2146828281
CTL_E_DIVISIONBYZERO: Final = -2146828277
CTL_E_OUTOFSTRINGSPACE: Final = -2146828274
CTL_E_OUTOFSTACKSPACE: Final = -2146828260
CTL_E_BADFILENAMEORNUMBER: Final = -2146828236
CTL_E_FILENOTFOUND: Final = -2146828235
CTL_E_BADFILEMODE: Final = -2146828234
CTL_E_FILEALREADYOPEN: Final = -2146828233
CTL_E_DEVICEIOERROR: Final = -2146828231
CTL_E_FILEALREADYEXISTS: Final = -2146828230
CTL_E_BADRECORDLENGTH: Final = -2146828229
CTL_E_DISKFULL: Final = -2146828227
CTL_E_BADRECORDNUMBER: Final = -2146828225
CTL_E_BADFILENAME: Final = -2146828224
CTL_E_TOOMANYFILES: Final = -2146828221
CTL_E_DEVICEUNAVAILABLE: Final = -2146828220
CTL_E_PERMISSIONDENIED: Final = -2146828218
CTL_E_DISKNOTREADY: Final = -2146828217
CTL_E_PATHFILEACCESSERROR: Final = -2146828213
CTL_E_PATHNOTFOUND: Final = -2146828212
CTL_E_INVALIDPATTERNSTRING: Final = -2146828195
CTL_E_INVALIDUSEOFNULL: Final = -2146828194
CTL_E_INVALIDFILEFORMAT: Final = -2146827967
CTL_E_INVALIDPROPERTYVALUE: Final = -2146827908
CTL_E_INVALIDPROPERTYARRAYINDEX: Final = -2146827907
CTL_E_SETNOTSUPPORTEDATRUNTIME: Final = -2146827906
CTL_E_SETNOTSUPPORTED: Final = -2146827905
CTL_E_NEEDPROPERTYARRAYINDEX: Final = -2146827903
CTL_E_SETNOTPERMITTED: Final = -2146827901
CTL_E_GETNOTSUPPORTEDATRUNTIME: Final = -2146827895
CTL_E_GETNOTSUPPORTED: Final = -2146827894
CTL_E_PROPERTYNOTFOUND: Final = -2146827866
CTL_E_INVALIDCLIPBOARDFORMAT: Final = -2146827828
CTL_E_INVALIDPICTURE: Final = -2146827807
CTL_E_PRINTERERROR: Final = -2146827806
CTL_E_CANTSAVEFILETOTEMP: Final = -2146827553
CTL_E_SEARCHTEXTNOTFOUND: Final = -2146827544
CTL_E_REPLACEMENTSTOOLONG: Final = -2146827542
CONNECT_E_FIRST: Final = -2147220992
CONNECT_E_LAST: Final = -2147220977
CONNECT_S_FIRST: Final = 262656
CONNECT_S_LAST: Final = 262671
CONNECT_E_NOCONNECTION: Final = -2147220992
CONNECT_E_ADVISELIMIT: Final = -2147220991
CONNECT_E_CANNOTCONNECT: Final = -2147220990
CONNECT_E_OVERRIDDEN: Final = -2147220989
CLASS_E_NOTLICENSED: Final = -2147221230
