class Quota:
    def __init__(self, bound, is_upper) -> None: ...
    @staticmethod
    def upper_bound(upper_bound): ...
    @staticmethod
    def lower_bound(lower_bound): ...
    def is_upper_bound(self): ...
    @property
    def bound(self): ...
    def is_acceptable(self, value): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
