import abc

class Serializer(metaclass=abc.ABCMeta):
    __meta__ = abc.ABCMeta
    def __init__(self, **config) -> None: ...
    @abc.abstractmethod
    def serialize(self, topic, value): ...
    def close(self) -> None: ...

class Deserializer(metaclass=abc.ABCMeta):
    __meta__ = abc.ABCMeta
    def __init__(self, **config) -> None: ...
    @abc.abstractmethod
    def deserialize(self, topic, bytes_): ...
    def close(self) -> None: ...
