from _typeshed import Incomplete

def format_workers(data: dict[str, Incomplete]) -> str: ...
def format_dirty(data: dict[str, Incomplete]) -> str: ...
def format_stats(data: dict[str, Incomplete]) -> str: ...
def format_listeners(data: dict[str, Incomplete]) -> str: ...
def format_config(data: dict[str, Incomplete]) -> str: ...
def format_help(data: dict[str, Incomplete]) -> str: ...
def format_all(data: dict[str, Incomplete]) -> str: ...
def format_response(command: str, data: dict[str, Incomplete]) -> str: ...
def run_command(socket_path: str, command: str, json_output: bool = False) -> int: ...
def run_interactive(socket_path: str, json_output: bool = False) -> int: ...
def main() -> int: ...
