class MetricName:
    def __init__(self, name, group, description=None, tags=None) -> None: ...
    @property
    def name(self): ...
    @property
    def group(self): ...
    @property
    def description(self): ...
    @property
    def tags(self): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
