from kafka.metrics.measurable_stat import AbstractMeasurableStat

class Total(AbstractMeasurableStat):
    def __init__(self, value: float = 0.0) -> None: ...
    def record(self, config, value, now) -> None: ...
    def measure(self, config, now): ...
