from _typeshed import Incomplete

from authlib.oauth1 import ResourceProtector as _ResourceProtector

class ResourceProtector(_ResourceProtector):
    app: Incomplete
    query_client: Incomplete
    query_token: Incomplete
    def __init__(self, app=None, query_client=None, query_token=None, exists_nonce=None) -> None: ...
    SUPPORTED_SIGNATURE_METHODS: Incomplete
    def init_app(self, app, query_client=None, query_token=None, exists_nonce=None): ...
    def get_client_by_id(self, client_id): ...
    def get_token_credential(self, request): ...
    def exists_nonce(self, nonce, request): ...
    def acquire_credential(self): ...
    def __call__(self, scope=None): ...

current_credential: Incomplete
