from _typeshed import Incomplete

class SortedSet:
    def __init__(self, iterable=None, key=None) -> None: ...
    def first(self): ...
    def last(self): ...
    def pop_last(self): ...
    def add(self, value): ...
    def remove(self, value): ...
    def __contains__(self, value) -> bool: ...
    def __iter__(self): ...
    __nonzero__: Incomplete
    __bool__: Incomplete
