from _typeshed import Incomplete

ZSTD_MAX_OUTPUT_SIZE: Incomplete
PYPY: Incomplete

def has_gzip(): ...
def has_snappy(): ...
def has_zstd(): ...
def has_lz4(): ...
def gzip_encode(payload, compresslevel=None): ...
def gzip_decode(payload): ...
def snappy_encode(payload, xerial_compatible: bool = True, xerial_blocksize=32768): ...
def snappy_decode(payload): ...

lz4_encode: Incomplete

def lz4f_decode(payload): ...

lz4_decode: Incomplete
lz4_decode = lz4f_decode

def lz4_encode_old_kafka(payload): ...
def lz4_decode_old_kafka(payload): ...
def zstd_encode(payload): ...
def zstd_decode(payload): ...
