class Sensor:
    def __init__(self, registry, name, parents, config, inactive_sensor_expiration_time_seconds) -> None: ...
    @property
    def name(self): ...
    @property
    def metrics(self): ...
    def record(self, value: float = 1.0, time_ms=None) -> None: ...
    def add_compound(self, compound_stat, config=None) -> None: ...
    def add(self, metric_name, stat, config=None) -> None: ...
    def has_expired(self): ...
